/**
 * @license
 * Copyright (c) 2020 Handsoncode. All rights reserved.
 */
import { SimpleCellAddress } from './Cell';
import { ColumnSearchStrategy } from './Lookup/SearchStrategy';
import { Config } from './Config';
import { ContentChanges } from './ContentChanges';
import { DateTimeHelper } from './DateTimeHelper';
import { DependencyGraph, Vertex } from './DependencyGraph';
import { FunctionRegistry } from './interpreter/FunctionRegistry';
import { InterpreterValue } from './interpreter/InterpreterValue';
import { NamedExpressions } from './NamedExpressions';
import { NumberLiteralHelper } from './NumberLiteralHelper';
import { Ast, RelativeDependency } from './parser';
import { Serialization } from './Serialization';
import { Statistics } from './statistics';
export declare class Evaluator {
    private readonly dependencyGraph;
    private readonly columnSearch;
    private readonly config;
    private readonly stats;
    readonly dateHelper: DateTimeHelper;
    private readonly numberLiteralsHelper;
    private readonly functionRegistry;
    private readonly namedExpressions;
    private readonly serialization;
    private interpreter;
    constructor(dependencyGraph: DependencyGraph, columnSearch: ColumnSearchStrategy, config: Config, stats: Statistics, dateHelper: DateTimeHelper, numberLiteralsHelper: NumberLiteralHelper, functionRegistry: FunctionRegistry, namedExpressions: NamedExpressions, serialization: Serialization);
    run(): void;
    partialRun(vertices: Vertex[]): ContentChanges;
    destroy(): void;
    runAndForget(ast: Ast, address: SimpleCellAddress, dependencies: RelativeDependency[]): InterpreterValue;
    /**
     * Recalculates formulas in the topological sort order
     */
    private recomputeFormulas;
    private evaluateAstToCellValue;
    private evaluateAstToRangeValue;
}
