/**
 * @license
 * Copyright (c) 2020 Handsoncode. All rights reserved.
 */
import { AbsoluteCellRange } from '../../AbsoluteCellRange';
import { InternalNoErrorScalarValue, InternalScalarValue, SimpleCellAddress } from '../../Cell';
import { ProcedureAst } from '../../parser';
import { ArgumentTypes, FunctionPlugin } from './FunctionPlugin';
import { SearchStrategy } from '../../Lookup/SearchStrategy';
export declare class LookupPlugin extends FunctionPlugin {
    private rowSearch;
    static implementedFunctions: {
        VLOOKUP: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: boolean;
            })[];
        };
        HLOOKUP: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: boolean;
            })[];
        };
        MATCH: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: number;
            })[];
        };
    };
    /**
     * Corresponds to VLOOKUP(key, range, index, [sorted])
     *
     * @param ast
     * @param formulaAddress
     */
    vlookup(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    /**
     * Corresponds to HLOOKUP(key, range, index, [sorted])
     *
     * @param ast
     * @param formulaAddress
     */
    hlookup(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    match(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    private doVlookup;
    private doHlookup;
    private doMatch;
    protected searchInRange(key: InternalNoErrorScalarValue, range: AbsoluteCellRange, sorted: boolean, searchStrategy: SearchStrategy): number;
}
