/**
 * @license
 * Copyright (c) 2020 Handsoncode. All rights reserved.
 */
import { InternalScalarValue, SimpleCellAddress } from '../../Cell';
import { ProcedureAst } from '../../parser';
import { ArgumentTypes, FunctionPlugin } from './FunctionPlugin';
export declare class RadixConversionPlugin extends FunctionPlugin {
    static implementedFunctions: {
        DEC2BIN: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        DEC2OCT: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        DEC2HEX: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        BIN2DEC: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        BIN2OCT: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        BIN2HEX: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        OCT2DEC: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        OCT2BIN: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        OCT2HEX: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        HEX2DEC: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        HEX2BIN: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        HEX2OCT: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
        DECIMAL: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            })[];
        };
        BASE: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue?: undefined;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
                minValue: number;
                maxValue: number;
            })[];
        };
    };
    dec2bin(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    dec2oct(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    dec2hex(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    bin2dec(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    bin2oct(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    bin2hex(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    oct2dec(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    oct2bin(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    oct2hex(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    hex2dec(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    hex2bin(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    hex2oct(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    base(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    decimal(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
}
