/**
 * @license
 * Copyright (c) 2020 Handsoncode. All rights reserved.
 */
import { InternalScalarValue, SimpleCellAddress } from '../../Cell';
import { ProcedureAst } from '../../parser';
import { ArgumentTypes, FunctionPlugin } from './FunctionPlugin';
/**
 * Interpreter plugin containing trigonometric functions
 */
export declare class TrigonometryPlugin extends FunctionPlugin {
    static implementedFunctions: {
        ACOS: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        ASIN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        COS: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        SIN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        TAN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        ATAN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        ATAN2: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        COT: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        SEC: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        CSC: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        SINH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        COSH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        TANH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        COTH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        SECH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        CSCH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        ACOT: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        ASINH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        ACOSH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        ATANH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        ACOTH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
    };
    /**
     * Corresponds to ACOS(value)
     *
     * Returns the arc cosine (or inverse cosine) of a number.
     *
     * @param ast
     * @param formulaAddress
     */
    acos(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    asin(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    cos(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    sin(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    tan(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    atan(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    atan2(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    cot(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    acot(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    sec(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    csc(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    sinh(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    asinh(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    cosh(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    acosh(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    tanh(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    atanh(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    coth(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    acoth(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    sech(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    csch(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
}
